@@ -58,7 +58,7 @@ class FlickrAuth(ConsumerBasedOAuth):
     def access_token(self, token):
         """Return request for access token value"""
         # Flickr is a bit different - it passes user information along with
-        # the access token, so temporarily store it to vie the user_data
+        # the access token, so temporarily store it to view the user_data
         # method easy access later in the flow!
         request = self.oauth_request(token, self.ACCESS_TOKEN_URL)
         response = self.fetch_response(request)
@@ -73,7 +73,7 @@ class FlickrAuth(ConsumerBasedOAuth):
                                                else None
         return token
 
-    def user_data(self, access_token):
+    def user_data(self, access_token, *args, **kwargs):
         """Loads user data from service"""
         return {
             'id': access_token.user_nsid,
@@ -81,6 +81,11 @@ class FlickrAuth(ConsumerBasedOAuth):
             'fullname': access_token.fullname,
         }
 
+    def auth_extra_arguments(self):
+        params = super(FlickrAuth, self).auth_extra_arguments() or {}
+        params['perms'] = 'read'
+        return params
+
 
 # Backend definition
 BACKENDS = {
