@@ -3,13 +3,15 @@ from django.db.utils import IntegrityError
 from social_auth.utils import setting
 from social_auth.models import UserSocialAuth
 from social_auth.backends.pipeline import warn_setting
+from social_auth.backends.exceptions import AuthException
+from django.utils.translation import ugettext
 
 
 def social_auth_user(backend, uid, user=None, *args, **kwargs):
     """Return UserSocialAuth account for backend/uid pair or None if it
     doesn't exists.
 
-    Raise ValueError if UserSocialAuth entry belongs to another user.
+    Raise AuthException if UserSocialAuth entry belongs to another user.
     """
     try:
         social_user = UserSocialAuth.objects.select_related('user')\
@@ -20,7 +22,10 @@ def social_auth_user(backend, uid, user=None, *args, **kwargs):
 
     if social_user:
         if user and social_user.user != user:
-            raise ValueError('Account already in use.', social_user)
+            raise AuthException(backend,
+                ugettext('This %(provider)s account is already in use.') % {
+                    'provider': backend.name
+                })
         elif not user:
             user = social_user.user
     return {'social_user': social_user, 'user': user}
@@ -54,5 +59,5 @@ def load_extra_data(backend, details, response, social_user, uid, user,
     if setting('SOCIAL_AUTH_EXTRA_DATA', True):
         extra_data = backend.extra_data(user, uid, response, details)
         if extra_data and social_user.extra_data != extra_data:
-            social_user.extra_data = extra_data
+            social_user.extra_data.update(extra_data)
             social_user.save()
